#!/bin/sh

# Default SWI-Prolog executable and linker names
DEFAULT_PROLOG=swipl
DEFAULT_PROLOG_LINKER=swipl-ld

# File where SWI-Prolog filenames and paths will be saved
FILE=.bplInfo

# Looks for SWI-Prolog executable
PROLOG_LIST="$DEFAULT_PROLOG swipl swiprolog pl prolog"
for X in $PROLOG_LIST; do
	which $X > /dev/null 2> /dev/null
	if [ $? -eq 0 ]; then
		# Ensures that the executable found is associated to SWI-Prolog
		# (on many systems, 'pl' actually starts the Perl interpreter)
		$X -v | grep SWI-Prolog > /dev/null
		if [ $? -eq 0 ]; then
			PROLOG=$X
			break
		fi
	fi
done

# Checks if we couldn't find SWI-Prolog executable
if [ -z $PROLOG ]; then
	echo
	echo "ERROR: SWI-Prolog executable couldn't be found."
	echo "Please modify the 'bplInfo.sh' script and set the DEFAULT_PROLOG variable"
	echo "with the command that starts SWI-Prolog in your system."
	echo "Example: DEFAULT_PROLOG=swipl"
	echo
	exit 1
fi

# Looks for SWI-Prolog linker
PROLOG_LINKER_LIST="$DEFAULT_PROLOG_LINKER swipl-ld plld"
for X in $PROLOG_LINKER_LIST; do
	which $X > /dev/null 2> /dev/null
	if [ $? -eq 0 ]; then
		PROLOG_LINKER=$X
		break
	fi
done

# Checks if we couldn't found SWI-Prolog linker
if [ -z $PROLOG ]; then
	echo
	echo "ERROR: SWI-Prolog linker couldn't be found."
	echo "Please modify the 'bplInfo.sh' script and set the DEFAULT_PROLOG_LINKER"
	echo "variable with the command that starts the SWI-Prolog linker in your"
	echo "system."
	echo "Example: DEFAULT_PROLOG_LINKER=swipl-ld"
	echo
	exit 1
fi

# Launches SWI-Prolog to get its home directory and saves
# all required information in the ".bplInfo" file
$PROLOG -g "current_prolog_flag(home, HomeDirectory), tell('$FILE'), \
            writef('%w %w %w\n', [$PROLOG, $PROLOG_LINKER, HomeDirectory]), told, halt."

# Writes script results
read PROLOG PROLOG_LINKER PLDIR < $FILE
echo -e "\tSWI-Prolog executable: $PROLOG"
echo -e "\tSWI-Prolog linker: $PROLOG_LINKER"
echo -e "\tSWI-Prolog home directory: $PLDIR"

