%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% On-line help for Bousi-Prolog shell commands

:- module(bplHelp, [
		bpl_help/0,             %
		command_help/1          % +Topic
   ]).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- set_prolog_flag(double_quotes, codes).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Predicates for displaying Bousi-Prolog on-line help
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% bpl_help
%
%     Shows a list with all available commands in Bousi-Prolog shell.
%

bpl_help :-
	write('----------------------------------------------------------------------------'), nl,
	write('Available commands:'), nl,
	write('ld ... (load) reads a source code file containing a program or an ontology,'), nl,
	write('       or prints the currently loaded files'), nl,
	write('sv ... (solve) executes a query [\'sv\' can be omitted]'), nl,
	write('lc ... (lambda-cut) prints or sets the lower bound for the approximation'), nl,
	write('       degree in weak unifications'), nl,
	write('pwd .. (print working directory) shows the path of the working directory'), nl,
	write('cd ... (change directory) changes the working directory'), nl,
	write('ls ... (list) lists the contents of the working directory'), nl,
	write('hp ... (help) shows this help text or information about a specific command'), nl,
	write('sh ... (shell) starts a new interactive shell process'), nl,
	write('ts ... (test) tests the system'), nl,
	write('bk ... (break) breaks to the underlying SWI-Prolog'), nl,
	write('qt ... (quit) quits the system'), nl,
	write('----------------------------------------------------------------------------'), nl,
	write('--          Enter \'hp <command>\' to get more help on a <command>          --'), nl,
	write('----------------------------------------------------------------------------'), nl.


%% command_help(+Command)
%
%     Displays help for a specific Bousi-Prolog command.
%

command_help(ld) :-
	write('----------------------------------------------------------------------------'), nl,
	write('LOAD'), nl,
	nl,
	write('Reads, compiles and loads a Bousi~Prolog program or ontology into the'), nl,
	write('database. The Bousi~Prolog system can have both a program and an ontology'), nl,
	write('loaded at the same time, but the program must always be loaded first and'), nl,
	write('the ontology is not allowed to contain clauses.'), nl,
	nl,
	write('The default file extensions are \'.bpl\' for programs and \'.ont\' for onto-'), nl,
	write('logies. If you want to load a file with a different extension, the full'), nl,
	write('filename must be provided.'), nl,
	nl,
	write('When used without an argument, this command shows the full path of the'), nl,
	write('program and ontology which are currently loaded (if any).'), nl,
	nl,
	write('Every Bousi~Prolog source code file is translated into an intermediate'), nl,
	write('Prolog representation of the original file called \'TPL code\'. This code is'), nl,
	write('stored in files which have the same name as the original BPL files but with'), nl,
	write('the \'.tpl\' extension. TPL files are used when BPL files haven\'t been modi-'), nl,
	write('fied since their corresponding TPL code was created. You can force the'), nl,
	write('rebuilding of the TPL files using the \'-f\' option.'), nl,
	nl,
	write('Syntax:'), nl,
	write('ld .............. shows the currently loaded program and ontology'), nl,
	write('ld <file> ....... loads a Bousi~Prolog program <file>'), nl,
	write('ld -f <file> .... rebuilds and loads a Bousi~Prolog program <file>'), nl,
	write('ld -o <file> .... loads a Bousi~Prolog ontology <file>'), nl,
	write('ld -fo <file> ... rebuilds and loads a Bousi~Prolog ontology <file>'), nl,
	write('----------------------------------------------------------------------------'), nl.

command_help(sv) :-
	write('----------------------------------------------------------------------------'), nl,
	write('SOLVE'), nl,
	nl,
	write('Executes a query about the currently loaded program using WSLD resolution.'), nl,
	write('Bousi~Prolog queries can contain any statement that is valid in the body of'), nl,
	write('a clause, including weak unifications and term comparisons. Queries can also'), nl,
	write('be executed if no program is loaded.'), nl,
	nl,
	write('When at least one solution is found for a query, the system shows its appro-'), nl,
	write('ximation degree and the resulting variable values (if any). You can then'), nl,
	write('type a semicolon (;) to look for more solutions or press RETURN to finish'), nl,
	write('the current query and return to the Bousi~Prolog shell.'), nl,
	nl,
	write('The \'sv\' command is the default Bousi~Prolog shell command. This means that'), nl,
	write('you can also enter queries without preceding them with \'sv\'.'), nl,
	nl,
	write('Syntax:'), nl,
	write('sv <query> ...... solves <query> using WSLD resolution'), nl,
	write('<query> ......... identical to \'sv <query>\''), nl,
	write('----------------------------------------------------------------------------'), nl.

command_help(lc) :-
	write('----------------------------------------------------------------------------'), nl,
	write('LAMBDA-CUT'), nl,
	nl,
	write('Prints or sets the lower bound allowed for the approximation degree in weak'), nl,
	write('unifications, which is known as the \'lambda-cut\'. When used to set a new'), nl,
	write('lambda-cut value, this command overrides the lambda_cut/1 directive that may'), nl,
	write('be present in the loaded program.'), nl,
	nl,
	write('The lambda-cut can be used to limit the expansion of the search space in a'), nl,
	write('WSLD resolution. When the lambda-cut value is set to a degree greater than'), nl,
	write('zero, the weak unification process will fail every time the computed approxi-'), nl,
	write('mation degree goes below the stored lambda-cut value. Therefore, the computa-'), nl,
	write('tion will also fail and all branches starting from that choice point will be'), nl,
	write('discarded.'), nl,
	nl,
	write('Syntax:'), nl,
	write('lc .............. prints the current lambda-cut value'), nl,
	write('lc <degree> ..... sets <degree> as the new lambda-cut value, which must be'), nl,
	write('                  a real number between 0 and 1 (inclusive)'), nl,
	write('----------------------------------------------------------------------------'), nl.

command_help(hp) :-
	write('----------------------------------------------------------------------------'), nl,
	write('HELP'), nl,
	nl,
	write('Gives a summary of the Bousi~Prolog shell commands or detailed information'), nl,
	write('about a specific command.'), nl,
	nl,
	write('Syntax:'), nl,
	write('hp .............. shows a summary of the shell commands'), nl,
	write('hp <command> .... shows information about the given <command>'), nl,
	write('----------------------------------------------------------------------------'), nl.

command_help(ls) :-
	write('----------------------------------------------------------------------------'), nl,
	write('LIST'), nl,
	nl,
	write('Lists the files and folders of the current working directory.'), nl,
	nl,
	write('Syntax:'), nl,
	write('ls .............. list the contents of the working directory'), nl,
	write('----------------------------------------------------------------------------'), nl.

command_help(pwd) :- 
	write('----------------------------------------------------------------------------'), nl,
	write('PRINT WORKING DIRECTORY'), nl,
	nl,
	write('Shows the absolute path of the current working directory.'), nl,
	nl,
	write('Syntax:'), nl,
	write('pwd ............. shows the full path of the working directory'), nl,
	write('----------------------------------------------------------------------------'), nl.

command_help(cd) :- 
	write('----------------------------------------------------------------------------'), nl,
	write('CHANGE DIRECTORY'), nl,
	nl,
	write('Changes the current working directory.'), nl,
	nl,
	write('Syntax:'), nl,
	write('cd <path> ....... changes the working directory to <path>, which can be'), nl,
	write('                  either an absolute or a relative path'), nl,
	write('----------------------------------------------------------------------------'), nl.

command_help(sh) :- 
	write('----------------------------------------------------------------------------'), nl,
	write('SHELL'), nl,
	nl,
	write('Starts a new interactive command-line shell without ending the Bousi~Prolog'), nl,
	write('system. The default shell is \'/bin/sh\' on Unix/Linux and \'cmd.exe\' on,'), nl,
	write('Windows, but you can override it entering a different path in the environ-'), nl,
	write('ment variables SHELL (on Unix/Linux) and COMSPEC (on Windows).'), nl,
	nl,
	write('Syntax:'), nl,
	write('sh .............. starts a new interactive shell process'), nl,
	write('----------------------------------------------------------------------------'), nl.

command_help(ts) :-
	write('----------------------------------------------------------------------------'), nl,
	write('TEST'), nl,
	nl,
	write('Test the system with a bundle of test programs.'), nl,
	nl,
	write('Syntax:'), nl,
	write('ts .............. tests the system'), nl,
	write('----------------------------------------------------------------------------'), nl.

command_help(bk) :-
	write('----------------------------------------------------------------------------'), nl,
	write('BREAK'), nl,
	nl,
	write('Leaves the Bousi~Prolog system and enter underlying Prolog. Use ?- main. to'), nl,
	write('restart the system.'), nl,
	nl,
	write('Syntax:'), nl,
	write('bk .............. breaks to the underlying SWI-Prolog'), nl,
	write('----------------------------------------------------------------------------'), nl.

command_help(qt) :-
	write('----------------------------------------------------------------------------'), nl,
	write('QUIT'), nl,
	nl,
	write('Quits the Bousi~Prolog system immediately. Always use this command instead'), nl,
	write('of the predefined predicate halt/0 when you want to end the application.'), nl,
	nl,
	write('Syntax:'), nl,
	write('qt .............. quits the system'), nl,
	write('----------------------------------------------------------------------------'), nl.

